@@ -85,7 +85,7 @@ PipeWireRecord::PipeWireRecord(QObject *parent)
     : QObject(parent)
     , d(new PipeWireRecordPrivate)
 {
-    d->m_encoder = "libx264rgb";
+    d->m_encoder = "libvpx";
     av_log_set_level(AV_LOG_DEBUG);
 }
 
@@ -220,7 +220,7 @@ void PipeWireRecordProduce::finish()
 
 QString PipeWireRecord::extension()
 {
-    return QStringLiteral("mp4");
+    return QStringLiteral("webm");
 }
 
 void PipeWireRecordProduce::setupStream()
@@ -229,8 +229,8 @@ void PipeWireRecordProduce::setupStream()
     disconnect(m_stream.get(), &PipeWireSourceStream::streamParametersChanged, this, &PipeWireRecordProduce::setupStream);
     avformat_alloc_output_context2(&m_avFormatContext, nullptr, nullptr, m_output.toUtf8().constData());
     if (!m_avFormatContext) {
-        qCWarning(PIPEWIRERECORD_LOGGING) << "Could not deduce output format from file: using MPEG." << m_output;
-        avformat_alloc_output_context2(&m_avFormatContext, nullptr, "mpeg", m_output.toUtf8().constData());
+        qCWarning(PIPEWIRERECORD_LOGGING) << "Could not deduce output format from file: using WebM." << m_output;
+        avformat_alloc_output_context2(&m_avFormatContext, nullptr, "webm", m_output.toUtf8().constData());
     }
     if (!m_avFormatContext) {
         qCDebug(PIPEWIRERECORD_LOGGING) << "could not set stream up";
@@ -265,13 +265,20 @@ void PipeWireRecordProduce::setupStream()
     }
     m_avCodecContext->time_base = AVRational{1, 1000};
 
-    if (avcodec_open2(m_avCodecContext, m_codec, nullptr) < 0) {
-        qCWarning(PIPEWIRERECORD_LOGGING) << "Could not open codec";
+    AVDictionary *options = nullptr;
+    av_dict_set_int(&options, "threads", 4, 0);
+    av_dict_set(&options, "preset", "ultrafast", 0);
+    av_dict_set(&options, "tune-content", "screen", 0);
+    av_dict_set(&options, "quality", "realtime", 0);
+
+    int ret = avcodec_open2(m_avCodecContext, m_codec, &options);
+    if (ret < 0) {
+        qCWarning(PIPEWIRERECORD_LOGGING) << "Could not open codec" << av_err2str(ret);
         return;
     }
 
     m_frame.reset(new CustomAVFrame);
-    int ret = m_frame->alloc(m_avCodecContext->width, m_avCodecContext->height, m_avCodecContext->pix_fmt);
+    ret = m_frame->alloc(m_avCodecContext->width, m_avCodecContext->height, m_avCodecContext->pix_fmt);
     if (ret < 0) {
         qCWarning(PIPEWIRERECORD_LOGGING) << "Could not allocate raw picture buffer" << av_err2str(ret);
         return;
@@ -321,16 +328,19 @@ void PipeWireRecordProduce::processFrame(const PipeWireFrame &frame)
     }
 
     if (frame.dmabuf) {
-        QImage qimage(m_stream->size(), QImage::Format_RGBA8888);
-        if (!m_dmabufHandler.downloadFrame(qimage, frame)) {
+        if (m_frameWithoutMetadataCursor.size() != m_stream->size()) {
+            m_frameWithoutMetadataCursor = QImage(m_stream->size(), QImage::Format_RGBA8888_Premultiplied);
+        }
+
+        if (!m_dmabufHandler.downloadFrame(m_frameWithoutMetadataCursor, frame)) {
             m_stream->renegotiateModifierFailed(frame.format, frame.dmabuf->modifier);
             return;
         }
-        updateTextureImage(qimage);
+        render(frame);
     } else if (frame.image) {
-        updateTextureImage(*frame.image);
+        updateTextureImage(*frame.image, frame);
     } else if (cursorChanged && !m_frameWithoutMetadataCursor.isNull()) {
-        render();
+        render(frame);
     }
 }
 
@@ -359,13 +369,30 @@ void PipeWireRecord::refresh()
     Q_EMIT stateChanged();
 }
 
-void PipeWireRecordProduce::updateTextureImage(const QImage &image)
+void PipeWireRecordProduce::updateTextureImage(const QImage &image, const PipeWireFrame &frame)
 {
     m_frameWithoutMetadataCursor = image;
-    render();
+    render(frame);
 }
 
-void PipeWireRecordProduce::render()
+static AVPixelFormat convertQImageFormatToAVPixelFormat(QImage::Format format)
+{
+    // Listing those handed by SpaToQImageFormat
+    switch (format) {
+    case QImage::Format_BGR888:
+        return AV_PIX_FMT_BGR24;
+    case QImage::Format_RGBX8888:
+    case QImage::Format_RGBA8888_Premultiplied:
+        return AV_PIX_FMT_RGBA;
+    case QImage::Format_RGB32:
+        return AV_PIX_FMT_RGB32;
+    default:
+        qDebug() << "Unexpected pixel format" << format;
+        return AV_PIX_FMT_RGB32;
+    }
+}
+
+void PipeWireRecordProduce::render(const PipeWireFrame &frame)
 {
     Q_ASSERT(!m_frameWithoutMetadataCursor.isNull());
 
@@ -382,7 +409,7 @@ void PipeWireRecordProduce::render()
     sws_context = sws_getCachedContext(sws_context,
                                        image.width(),
                                        image.height(),
-                                       AV_PIX_FMT_RGB32,
+                                       convertQImageFormatToAVPixelFormat(image.format()),
                                        m_avCodecContext->width,
                                        m_avCodecContext->height,
                                        m_avCodecContext->pix_fmt,
@@ -392,8 +419,8 @@ void PipeWireRecordProduce::render()
                                        nullptr);
     sws_scale(sws_context, buffers, strides, 0, m_avCodecContext->height, m_frame->m_avFrame->data, m_frame->m_avFrame->linesize);
 
-    if (auto v = m_stream->currentPresentationTimestamp(); v.has_value()) {
-        const auto current = std::chrono::duration_cast<std::chrono::milliseconds>(v.value()).count();
+    if (frame.presentationTimestamp.has_value()) {
+        const auto current = std::chrono::duration_cast<std::chrono::milliseconds>(*frame.presentationTimestamp).count();
         if ((*m_avFormatContext->streams)->start_time == 0) {
             (*m_avFormatContext->streams)->start_time = current;
         }
@@ -404,7 +431,20 @@ void PipeWireRecordProduce::render()
         m_frame->m_avFrame->pts = AV_NOPTS_VALUE;
     }
 
+    // Let's add a key frame every 100 frames and also the first 5 frames
+    if (frame.sequential && (*frame.sequential == 0 || (*frame.sequential - m_lastKeyFrame) > 100)) {
+        m_frame->m_avFrame->key_frame = 1;
+        m_lastKeyFrame = frame.sequential.has_value() ? *frame.sequential : 11;
+    }
+
+    if (m_lastPts > 0 && m_frame->m_avFrame->pts <= m_lastPts) {
+        // Make sure we don't have two frames at the same presentation time
+        m_frame->m_avFrame->pts = m_lastPts + 1;
+    }
+    m_lastPts = m_frame->m_avFrame->pts;
+
     const int ret = avcodec_send_frame(m_avCodecContext, m_frame->m_avFrame);
+    // qDebug() << "issued" << m_frame->m_avFrame->pts;
     if (ret < 0) {
         qCWarning(PIPEWIRERECORD_LOGGING) << "Error sending a frame for encoding:" << av_err2str(ret);
         return;
